//! # Independent push notification service
//!
//! @author Pierre Hubert


use actix_web::http::{HeaderValue, StatusCode};
use reqwest::blocking::Client;
use reqwest::header::HeaderMap;
use serde::{Deserialize, Serialize};

use crate::data::config::{conf, IndependentPushService};
use crate::data::error::{ExecError, Res};

#[derive(Deserialize)]
struct CreateTokenResult {
    token: String
}

#[derive(Serialize)]
struct RemoveTokenRequest {
    client: String,
}

fn svc_conf() -> Res<&'static IndependentPushService> {
    match &conf().independent_push_service {
        Some(c) => Ok(c),
        None => Err(ExecError::boxed_new("Missing push configuration!")),
    }
}

fn create_client() -> Res<Client> {
    let mut headers = HeaderMap::new();
    headers.insert("token", HeaderValue::from_str(&svc_conf()?.control_token.clone())?);
    Ok(Client::builder().default_headers(headers).build()?)
}

/// Create a new client token
pub fn create_token() -> Res<String> {
    let client = create_client()?
        .get(&format!("{}{}", svc_conf()?.control_url, "create_client"))
        .send()?;


    let response: CreateTokenResult = client.json()?;

    Ok(response.token)
}

/// Destroy a client token
pub fn remove_token(t: &str) -> Res {
    let client = create_client()?
        .get(&format!("{}{}", svc_conf()?.control_url, "remove_client"))
        .json(&RemoveTokenRequest { client: t.to_string() })
        .send()?;

    if client.status() == StatusCode::OK {
        return Ok(());
    }

    Err(ExecError::boxed_string(
        format!("Failed to remove client, got a status {} for service!", client.status().as_u16())))
}